import grok
from zope.component.interfaces import IFactory
from zope.component import createObject
from zope.interface import implementedBy
from waeup.interfaces import IFaculty, IDepartment
from waeup.viewlets import (MainArea, LeftSidebar, Index, FormWrapMixin,
                            Manage, Add)

class Faculty(grok.Container):
    grok.implements(IFaculty)

    def __init__(self,
                 title=u'Unnamed Faculty',
                 title_prefix=u'faculty',
                 code=u'NA', **kw):
        super(Faculty, self).__init__(**kw)
        self.title = title
        self.title_prefix = title_prefix
        self.code = code

    def addDepartment(self, department):
        if not IDepartment.providedBy(department):
            raise TypeError('Faculties contain only IDepartment instances')
        self[department.code] = department

    def clear(self):
        keys = self.keys()
        for key in keys:
            del self[key]

        
class FacultyFactory(grok.GlobalUtility):
    """A factory for faculty containers.
    """
    grok.implements(IFactory)
    grok.name(u'waeup.Faculty')
    title = u"Create a new faculty.",
    description = u"This factory instantiates new faculty instances."

    def __call__(self, *args, **kw):
        return Faculty()

    def getInterfaces(self):
        return implementedBy(Faculty)

class DisplayForm(grok.DisplayForm):
    grok.context(IFaculty)
    form_fields = grok.AutoFields(IFaculty)
 
class Content(FormWrapMixin, grok.Viewlet):
    """A viewlet that wraps the `AddFacultyForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(IFaculty)
    grok.view(Index)
    grok.order(2)
    formview_name = 'displayform' # The name of the formview we want
                                  # to be rendered in this viewlet.
    
class DepartmentList(grok.Viewlet):
    grok.viewletmanager(MainArea)
    grok.context(IFaculty)
    grok.view(Index)
    grok.order(3)

    def getDepartments(self):
        """Convinience method to create a sorted list of departments.

        It provides a list of dicts with entries for all data needed by
        usual list templates.
        """
        result = []
        for key, val in self.context.items():
            result.append(dict(id=key, name=val.title))
        return result


class ManageForm(grok.EditForm):
    """Manage the basic properties of a `Faculty` instance.
    """

    form_fields = grok.AutoFields(IFaculty)

    @property
    def label(self):
        # Set faculty name as form label
        return self.form_fields['title'].field.get(self.context)
    
    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel')
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

class ManageFaculty(FormWrapMixin, grok.Viewlet):
    """A viewlet that renders the `ManageForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(IFaculty)
    grok.view(Manage)
    grok.require('waeup.manageUniversity')
    
    formview_name = 'manageform' # The name of the formview we want to
                                 # be rendered in this viewlet.

class ManageLink(grok.Viewlet):
    grok.viewletmanager(LeftSidebar)
    grok.context(IFaculty)
    grok.view(Index)
    grok.order(5)
    # This link is only displayed, if the user is allowed to use it!
    grok.require('waeup.manageUniversity')
    
    def render(self):
        return u'<div class="portlet"><a href="manage">Settings</a></div>'

class AddDepartmentForm(grok.AddForm):
    grok.context(IFaculty)
    form_fields = grok.AutoFields(IDepartment)
    label = 'Add a department'

    @grok.action('Add department')
    def addDepartment(self, **data):
        department = createObject(u'waeup.Department')
        self.applyData(department, **data)
        try:
            self.context.addDepartment(department)
        except DuplicationError:
            self.status = Invalid('The name chosen already exists '
                                  'in the database')
            return
        self.redirect(self.url(self.context))

        
class AddDepartment(FormWrapMixin, grok.Viewlet):
    """A viewlet that wraps the `AddDepartmentForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(IFaculty)
    grok.view(Add)
    grok.require('waeup.manageUniversity')

    formview_name = 'adddepartmentform' # The name of the formview we
                                     # want to be rendered in this
                                     # viewlet.

class AddDepartmentLink(grok.Viewlet):
    """A link in the left sidebar displaying 'Add department'
    """
    grok.viewletmanager(LeftSidebar)
    grok.context(IFaculty)
    grok.view(Index)
    grok.order(5)
    # This is so cool! This link is only displayed, when the user is
    # allowed to use it!
    grok.require('waeup.manageUniversity')
    
    def render(self):
        return u'<div class="portlet"><a href="add">Add department</a></div>'


